/*
 * $Workfile: gfx_vip.c $
 *
 * This file contains routines to control the video input port (VIP).
 * 
 *    gfx_set_vip_enable
 *    gfx_set_vip_capture_run_mode
 *    gfx_set_vip_base
 *    gfx_set_vip_pitch
 *    gfx_set_vip_mode
 *    gfx_set_vbi_enable
 *    gfx_set_vbi_mode
 *    gfx_set_vbi_base
 *    gfx_set_vbi_pitch
 *    gfx_set_vbi_direct
 *    gfx_set_vbi_interrupt
 *    gfx_set_vip_bus_request_threshold_high
 *    gfx_set_vip_last_line
 *    gfx_test_vip_odd_field
 *    gfx_test_vip_bases_updated
 *    gfx_test_vip_fifo_overflow
 *    gfx_get_vip_line
 *    gfx_get_vip_base
 *    gfx_get_vbi_pitch
 *
 * And the following routines if GFX_READ_ROUTINES is set:
 *
 *    gfx_get_vip_enable
 *    gfx_get_vip_pitch
 *    gfx_get_vip_mode
 *    gfx_get_vbi_enable
 *    gfx_get_vbi_mode
 *    gfx_get_vbi_base
 *    gfx_get_vbi_direct
 *    gfx_get_vbi_interrupt
 *    gfx_get_vip_bus_request_threshold_high
 *
 * NSC_COPYRIGHT
 *
 * Copyright (c) 2003 National Semiconductor Corporation ("NSC").
 *
 * All Rights Reserved.  Unpublished rights reserved under the
 * copyright laws of the United States of America, other countries
 * and international treaties.  The software is provided without
 * fee.  Permission to use, copy, store, modify, disclose, transmit
 * or distribute the software is granted, provided that this
 * copyright notice must appear in any copy, modification,
 * disclosure, transmission or distribution of the software.
 *
 * NSC retains all ownership, copyright, trade secret and
 * proprietary rights in the software. THIS SOFTWARE HAS BEEN
 * PROVIDED "AS IS," WITHOUT EXPRESS OR IMPLIED WARRANTY INCLUDING,
 * WITHOUT LIMITATION, IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR USE AND NON-INFRINGEMENT.
 *
 * NSC does not assume or authorize any other person to assume for
 * it any liability in connection with the Software. NSC SHALL NOT
 * BE LIABLE TO COMPANY, OR ANY THIRD PARTY, IN CONTRACT, TORT,
 * WARRANTY, STRICT LIABILITY, OR OTHERWISE FOR ANY DIRECT DAMAGES,
 * OR FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES, INCLUDING BUT NOT LIMITED TO, BUSINESS INTERRUPTION,
 * LOST PROFITS OR GOODWILL, OR LOSS OF INFORMATION EVEN IF NSC IS
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * END_NSC_COPYRIGHT 
 */


/* INCLUDE SUPPORT FOR SC1200, IF SPECIFIED. */

#if GFX_VIP_SC1200
#include "vip_1200.c"
#endif

/* WRAPPERS IF DYNAMIC SELECTION */
/* Extra layer to call either CS5530 or SC1200 routines. */

#if GFX_VIP_DYNAMIC

/*-----------------------------------------------------------------------------
 * gfx_set_vip_enable
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_enable(int enable)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_enable(enable);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_capture_run_mode
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_capture_run_mode(int mode)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_capture_run_mode(mode);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_base
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_base(unsigned long even, unsigned long odd)
{	
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_base(even, odd);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_pitch
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_pitch(unsigned long pitch)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_pitch(pitch);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_mode
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_mode(int mode)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_mode(mode);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_enable
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_enable(int enable)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_enable(enable);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_mode
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_mode(int mode)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_mode(mode);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_base
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_base(unsigned long even, unsigned long odd)
{	
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_base(even, odd);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_pitch
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_pitch(unsigned long pitch)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_pitch(pitch);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_direct
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_direct(unsigned long even_lines, unsigned long odd_lines)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_direct(even_lines, odd_lines);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vbi_interrupt
 *-----------------------------------------------------------------------------
 */
int gfx_set_vbi_interrupt(int enable)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vbi_interrupt(enable);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_bus_request_threshold_high
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_bus_request_threshold_high(int enable)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_bus_request_threshold_high(enable);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_set_vip_last_line
 *-----------------------------------------------------------------------------
 */
int gfx_set_vip_last_line(int last_line)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_set_vip_last_line(last_line);
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_test_vip_odd_field
 *-----------------------------------------------------------------------------
 */
int gfx_test_vip_odd_field(void)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_test_vip_odd_field();
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_test_vip_bases_updated
 *-----------------------------------------------------------------------------
 */
int gfx_test_vip_bases_updated(void)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_test_vip_bases_updated();
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_test_vip_fifo_overflow
 *-----------------------------------------------------------------------------
 */
int gfx_test_vip_fifo_overflow(void)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_test_vip_fifo_overflow();
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vip_line
 *-----------------------------------------------------------------------------
 */
int gfx_get_vip_line(void)
{
	int status = GFX_STATUS_UNSUPPORTED;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		status = sc1200_get_vip_line();
#endif
	return(status);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vip_base
 *-----------------------------------------------------------------------------
 */
unsigned long gfx_get_vip_base(int odd)
{
	unsigned long base = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		base = sc1200_get_vip_base(odd);
#endif
	return(base);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_pitch
 *-----------------------------------------------------------------------------
 */
unsigned long gfx_get_vbi_pitch(void)
{
	unsigned long pitch = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		pitch = sc1200_get_vbi_pitch();
#endif
	return(pitch);
}

/*************************************************************/
/*  READ ROUTINES  |  INCLUDED FOR DIAGNOSTIC PURPOSES ONLY  */
/*************************************************************/

#if GFX_READ_ROUTINES

/*-----------------------------------------------------------------------------
 * gfx_get_vip_enable
 *-----------------------------------------------------------------------------
 */
int gfx_get_vip_enable(void)
{
	int enable = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		enable = sc1200_get_vip_enable();
#endif
	return(enable);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vip_pitch
 *-----------------------------------------------------------------------------
 */
unsigned long gfx_get_vip_pitch(void)
{
	unsigned long pitch = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		pitch = sc1200_get_vip_pitch();
#endif
	return(pitch);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vip_mode
 *-----------------------------------------------------------------------------
 */
int gfx_get_vip_mode(void)
{
	int mode = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		mode = sc1200_get_vip_mode();
#endif
	return(mode);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_enable
 *-----------------------------------------------------------------------------
 */
int gfx_get_vbi_enable(void)
{
	int enable = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		enable = sc1200_get_vbi_enable();
#endif
	return(enable);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_mode
 *-----------------------------------------------------------------------------
 */
int gfx_get_vbi_mode(void)
{
	int mode = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		mode = sc1200_get_vbi_mode();
#endif
	return(mode);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_base
 *-----------------------------------------------------------------------------
 */
unsigned long gfx_get_vbi_base(int odd)
{
	unsigned long base = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		base = sc1200_get_vbi_base(odd);
#endif
	return(base);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_direct
 *-----------------------------------------------------------------------------
 */
unsigned long gfx_get_vbi_direct(int odd)
{
	unsigned long vbi_direct_lines = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		vbi_direct_lines = sc1200_get_vbi_direct(odd);
#endif
	return(vbi_direct_lines);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vbi_interrupt
 *-----------------------------------------------------------------------------
 */
int gfx_get_vbi_interrupt(void)
{
	int enable = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		enable = sc1200_get_vbi_interrupt();
#endif
	return(enable);
}

/*-----------------------------------------------------------------------------
 * gfx_get_vip_bus_request_threshold_high
 *-----------------------------------------------------------------------------
 */
int gfx_get_vip_bus_request_threshold_high(void)
{
	int enable = 0;
#if GFX_VIP_SC1200
	if (gfx_vip_type == GFX_VIP_TYPE_SC1200)
		enable = sc1200_get_vip_bus_request_threshold_high();
#endif
	return(enable);
}

#endif /* GFX_READ_ROUTINES */

#endif /* GFX_VIP_DYNAMIC */

/* END OF FILE */
