/*
 * $Workfile: nsc_gx2_cursor.c $
 * $Revision: 1 $
 * $Author: Johnz $
 *
 * File Contents: Xfree cursor implementation routines
 *                for geode HWcursor init.setting cursor color,image etc
 *                are done here.
 * Project:       Geode Xfree Frame buffer device driver.
 *
 */

/* 
 * NSC_LIC_COPYRIGHT
 *
 * Copyright (c) 2001-2003 National Semiconductor Corporation ("NSC").
 *
 * All Rights Reserved.  Unpublished rights reserved under the copyright 
 * laws of the United States of America, other countries and international 
 * treaties.  The software is provided without fee.  Permission to use, 
 * copy, store, modify, disclose, transmit or distribute the software is 
 * granted, provided that this copyright notice must appear in any copy, 
 * modification, disclosure, transmission or distribution of the software.
 *  
 * NSC retains all ownership, copyright, trade secret and proprietary rights 
 * in the software. 
 * THIS SOFTWARE HAS BEEN PROVIDED "AS IS," WITHOUT EXPRESS OR IMPLIED 
 * WARRANTY INCLUDING, WITHOUT LIMITATION, IMPLIED WARRANTIES OF 
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR USE AND NON-INFRINGEMENT.
 *
 * NSC does not assume or authorize any other person to assume for it any 
 * liability in connection with the Software. NSC SHALL NOT BE LIABLE TO 
 * COMPANY, OR ANY THIRD PARTY, IN CONTRACT, TORT, WARRANTY, STRICT 
 * LIABILITY, OR OTHERWISE FOR ANY DIRECT DAMAGES, OR FOR ANY SPECIAL, 
 * INDIRECT, INCIDENTAL OR CONSEQUENTIAL DAMAGES, INCLUDING BUT NOT 
 * LIMITED TO, BUSINESS INTERRUPTION, LOST PROFITS OR GOODWILL, OR LOSS 
 * OF INFORMATION EVEN IF NSC IS ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * END_NSC_LIC_COPYRIGHT */

#include "xf86.h"
#include "xf86_OSproc.h"
#include "xf86_ansic.h"
#include "xf86Pci.h"
#include "xf86PciInfo.h"
#include "nsc.h"

/* Forward declarations of the functions */
Bool GX2HWCursorInit(ScreenPtr pScreen);
static void GX2SetCursorColors(ScrnInfoPtr pScreenInfo, int bg, int fg);
static void GX2SetCursorPosition(ScrnInfoPtr pScreenInfo, int x, int y);
void GX2LoadCursorImage(ScrnInfoPtr pScreenInfo, unsigned char *src);
void GX2HideCursor(ScrnInfoPtr pScreenInfo);
void GX2ShowCursor(ScrnInfoPtr pScreenInfo);
static Bool GX2UseHWCursor(ScreenPtr pScreen, CursorPtr pCurs);
extern void GX2SetVideoPosition(int x, int y, int width, int height,
				short src_w, short src_h, short drw_w,
				short drw_h, int id, int offset,
				ScrnInfoPtr pScrn);

/*----------------------------------------------------------------------------
 * GX2HWCursorInit.
 *
 * Description	:This function sets the cursor information by probing the
 * hardware.
 *
 * Parameters.
 *     pScreen	:Screeen pointer structure.
 *
 * Returns		:TRUE on success and FALSE on Failure
 *
 * Comments		:Geode supports the hardware_cursor,no need to enable SW
 *                    cursor.
*----------------------------------------------------------------------------
*/
Bool
GX2HWCursorInit(ScreenPtr pScreen)
{
   ScrnInfoPtr pScreenInfo = xf86Screens[pScreen->myNum];
   GeodePtr pGeode = GEODEPTR(pScreenInfo);
   xf86CursorInfoPtr infoPtr;

   infoPtr = xf86CreateCursorInfoRec();
   if (!infoPtr)
      return FALSE;
   /* the geode structure is intiallized with the cursor infoRec */
   pGeode->CursorInfo = infoPtr;
   infoPtr->MaxWidth = 32;
   infoPtr->MaxHeight = 32;
   /* seeting up the cursor flags */
   infoPtr->Flags = HARDWARE_CURSOR_BIT_ORDER_MSBFIRST |
	 HARDWARE_CURSOR_TRUECOLOR_AT_8BPP |
	 HARDWARE_CURSOR_SOURCE_MASK_NOT_INTERLEAVED;
   /* cursor info ptr is intiallized with the values obtained from
    * * durnago calls
    */
   infoPtr->SetCursorColors = GX2SetCursorColors;
   infoPtr->SetCursorPosition = GX2SetCursorPosition;
   infoPtr->LoadCursorImage = GX2LoadCursorImage;
   infoPtr->HideCursor = GX2HideCursor;
   infoPtr->ShowCursor = GX2ShowCursor;
   infoPtr->UseHWCursor = GX2UseHWCursor;
   return (xf86InitCursor(pScreen, infoPtr));
}

/*----------------------------------------------------------------------------
 * GX2SetCursorColors.
 *
 * Description	:This function sets the cursor foreground and background
 *                    colors
 * Parameters:
 *    pScreen:	Screeen pointer structure.
 *    	   bg:	Specifies the color value of cursor background color.
 *    	   fg:	Specifies the color value of cursor foreground color.
 *    Returns:	none.
 *
 *   Comments:	The integer color value passed by this function is
 *              converted into  * RGB  value by the gfx_set_color routines.
 *----------------------------------------------------------------------------
 */
static void
GX2SetCursorColors(ScrnInfoPtr pScreenInfo, int bg, int fg)
{
   GFX(set_cursor_colors(bg, fg));
}

/*----------------------------------------------------------------------------
 * GX2SetCursorPosition.
 *
 * Description	:This function sets the cursor co -ordinates and enable the
 *               cursor.
 *
 * Parameters:
 *    pScreen: Screeen pointer structure.
 *    	    x: Specifies the x-cordinates of the cursor.
 *    	    y: Specifies the y co-ordinate of the cursor.
 *    Returns: none.
 *
 *----------------------------------------------------------------------------
 */
static void
GX2SetCursorPosition(ScrnInfoPtr pScreenInfo, int x, int y)
{
   static unsigned long panOffset = 0;
   GeodePtr pGeode = GEODEPTR(pScreenInfo);

   unsigned short xhot = 0, yhot = 0;

   if (x < 0) {
      xhot = (unsigned short)(-x);
      x = 0;
   }
   if (y < 0) {
      yhot = (unsigned short)(-y);
      y = 0;
   }

   GFX(set_cursor_position(pGeode->CursorStartOffset, x, y, xhot, yhot));
   GFX(set_cursor_enable(1));

   if ((pGeode->OverlayON) && (pGeode->Panel)) {
#if defined(STB_X)
      Gal_get_display_offset(&pGeode->PrevDisplayOffset);
#else
      pGeode->PrevDisplayOffset = gfx_get_display_offset();
#endif
      if (pGeode->PrevDisplayOffset != panOffset) {
	 GX2SetVideoPosition(pGeode->video_x, pGeode->video_y,
			     pGeode->video_w, pGeode->video_h,
			     pGeode->video_srcw, pGeode->video_srch,
			     pGeode->video_dstw, pGeode->video_dsth,
			     pGeode->video_id, pGeode->video_offset,
			     pGeode->video_scrnptr);
	 panOffset = pGeode->PrevDisplayOffset;
      }
   }
}

/*----------------------------------------------------------------------------
 * GX2LoadCursorImage
 *
 * Description	:This function loads the 32x32 cursor pattern.The shape
 *               and color is set by AND and XOR masking of arrays of 32
 *               DWORD.
 * Parameters:
 *    pScreen: Screeen pointer structure.
 *    src    : Specifies cursor data.
 * Returns   : none
 *
 *----------------------------------------------------------------------------
*/
void
GX2LoadCursorImage(ScrnInfoPtr pScreenInfo, unsigned char *src)
{
   int i, j;
   unsigned long shape;
   unsigned long mask;
   unsigned long andMask[32] = { 0, };
   unsigned long xorMask[32] = { 0, };
   GeodePtr pGeode = GEODEPTR(pScreenInfo);

   j = 0;
   for (i = 0; i < 32; i++) {
      if (src) {
	 shape = ((unsigned long)src[i * 4] << 24) |
	       ((unsigned long)src[i * 4 + 1] << 16) |
	       ((unsigned long)src[i * 4 + 2] << 8) |
	       ((unsigned long)src[i * 4 + 3] << 0);
	 mask = ((unsigned long)src[i * 4 + 128] << 24) |
	       ((unsigned long)src[i * 4 + 1 + 128] << 16) |
	       ((unsigned long)src[i * 4 + 2 + 128] << 8) |
	       ((unsigned long)src[i * 4 + 3 + 128] << 0);
      } else {
	 mask = 0x0;
	 shape = 0xFFFFFFFF;
      }

      andMask[i] = ~(mask);
      xorMask[i] = shape & mask;
   }

   GFX(set_cursor_shape32(pGeode->CursorStartOffset, andMask, xorMask));
}

/*----------------------------------------------------------------------------
 * GX2HideCursor.
 *
 * Description	:This function will disable the cursor.
 *
 * Parameters:
 *    pScreen: Handles to the Screeen pointer structure.
 *
 *    Returns: none.
 *
 *   Comments:	gfx_set_cursor enable function is hardcoded to disable
 *		the cursor.
 *----------------------------------------------------------------------------
 */
void
GX2HideCursor(ScrnInfoPtr pScreenInfo)
{
   GFX(set_cursor_enable(0));
}

/*----------------------------------------------------------------------------
 * GX2ShowCursor
 *
 * Description	:This function will enable  the cursor.
 *
 * Parameters:
 *	pScreen		:Handles to the Screeen pointer structure.
 *
 * Returns      :none
 *
 * Comments		:gfx_set_cursor enable function is hardcoded to enable the
 * 											cursor
 *----------------------------------------------------------------------------
*/
void
GX2ShowCursor(ScrnInfoPtr pScreenInfo)
{
   GFX(set_cursor_enable(1));
}

/*----------------------------------------------------------------------------
 * GX2UseHwCursor.
 *
 * Description	:This function will sets the hardware cursor flag in
 *                 pscreen  structure.
 *
 * Parameters.
 *	pScreen		:Handles to the Screeen pointer structure.
 *
 * Returns		:none
 *
 * Comments		:none
 *
 *----------------------------------------------------------------------------
*/
static Bool
GX2UseHWCursor(ScreenPtr pScreen, CursorPtr pCurs)
{
   ScrnInfoPtr pScreenInfo = XF86SCRNINFO(pScreen);

   if (pScreenInfo->currentMode->Flags & V_DBLSCAN)
      return FALSE;
   return TRUE;
}

/* End of File */
